


local Error = {}

Error.__index = Error

local function new_error(msg, msg_with_line, status, traceback)
    print('new_error', msg, msg_with_line)
    local ret = {
        msg = msg,
        msg_with_line = msg_with_line,
        status = status,
        traceback = traceback,
    }
    setmetatable(ret, Error)
    return ret
end
---Wrapper around assert that coverts the
---message into an pipe sepereted list
---this format will be used by pcall to reconstruct
---an Error if any calls to assert raise an error
---@param test boolean
---@param msg string
---@param status number defualts to 500
function Error.assert(test, msg, status)
    msg = msg or 'assertion failed'
    local traceback = debug.traceback(msg, 2)
    local info = debug.getinfo(2)
    local orig_loc = string.format("%s:%s", info.short_src or '', info.currentline or '')
    msg = string.format("%s|%i|%s|%s", msg, status or 500, traceback, orig_loc)
    return assert(test, msg)
end

local function parse_error_msg(s)
    local i = 1
    local keys = {
        'msg',
        'status',
        'traceback',
        'orig_loc'
    }
    local values = {}
    for part in string.gmatch(s, "[^|]+") do
        values[keys[i]] = part
        i = i + 1
    end
    if i ~= 5 then
        return s
    else
        return values
    end
end

function Error.pcall(...)
    local res = table.pack(pcall(...))
    local success = res[1]
    if not success then
        local parsed = parse_error_msg(res[2])
        if type(parsed) == 'string' then
            return success, parsed
        end
        local stripped_message = string.gsub(parsed.msg, '^(.+luxure/error.lua:[0-9]+): ', '')
        local status = math.tointeger(parsed.status) or 500
        return false, new_error(
            stripped_message,
            string.format('%s%s', parsed.orig_loc, stripped_message),
            status,
            parsed.traceback
        )
    end
    res.n = nil
    return success, table.unpack(res, 2)
end

return {
    Error = Error,
}
